import argparse
import wikipedia_tools.scraper.revision_extractor as re
import wikipedia_tools.scraper.category_extractor as ce
import json
from functools import reduce

REVISIONS_FILE_POSTFIX = "_revisions.jsonl"


def read_pages_file(pages_file):
    with open(pages_file) as f:
        lines = f.readlines()

    return lines


def main():
    parser = argparse.ArgumentParser(
        prog="Wikipedia Parser", description="Parser to extract Wikipedia discourse."
    )

    parser.add_argument(
        "-c", "--categories", type=str, nargs="*", help="Categories to parse."
    )

    parser.add_argument(
        "-p",
        "--pages_file",
        type=str,
        help="File with a list of pages to parse. One page per line.",
    )

    parser.add_argument(
        "-o",
        "--outfile_prefix",
        type=str,
        required=True,
        help="Prefix for output files.",
    )

    parser.add_argument(
        "-g",
        action="store_true",
        help="Flag indicating if a revision and pagelink graph should be extracted",
    )

    args = parser.parse_args()

    if args.pages_file:
        pages = read_pages_file(args.pages_file)
    else:
        pages = []

    if args.categories:
        pages = set(
            pages
            + reduce(list.__add__, [ce.get_category_pages(c) for c in args.categories])
        )

    if len(pages) == 0:
        raise Exception(
            "No pages to parse. Please specify a proper category or a file with page names."
        )
    else:
        print("parsing " + str(len(pages)) + "...")
        revisions_df = re.get_revisions_all_pages(pages, "en")

        with open(args.outfile_prefix + REVISIONS_FILE_POSTFIX, "w") as f:
            for r in revisions_df:
                f.write(json.dumps(r) + "\n")


if __name__ == "__main__":
    main()



import dataclasses

import pandas as pd

from wikipedia_tools.wikipedia_tools.processor import processor
from wikipedia_tools.utils import properties


@dataclasses.dataclass
class WikipediaRevisionAnalyzer:
    categories: list
    revisions_from: str = "01.01.2001"
    revisions_to: "str|None" = None
    corpus: str = properties.CORPUS
    download: bool = False

    def __post_init__(self):
        self.wikipages_revisions: pd.DataFrame = self.load_data()

    def get_edited_pages_count(self, period_type: str = ""):
        wikipedia_pages_per_category = processor.get_wikipedia_page_titles(
            categories=self.categories, corpus=self.corpus
        )
        return

    def get_revisions_count(self):
        return

    def get_revisions_per_user_type_count(self):
        return

    def get_edited_pages_per_user_type_count(self):
        return

    def get_user_type_count(self):
        return



import pandas as pd
import seaborn as sns
import matplotlib.pyplot as plt
import matplotlib
import ast

from wikipedia_tools.processor import processor
from wikipedia_tools.utils import properties, utils

import textwrap


# Helpers
def barplot(x, y, data, xlabel="Year", ylabel="Count", hue=None, title="", dodge=True):
    sns.set(font_scale=1)
    plt.style.use("fivethirtyeight")
    palette = (
        [sns.color_palette("PuBuGn_d")[0], sns.color_palette("PuBuGn_d")[-3]]
        if hue
        else [sns.color_palette("PuBuGn_d")[0]]
    )
    fig, axes = plt.subplots(figsize=(7, 5))
    sns.barplot(
        x=x,
        y=y,
        data=data,
        palette=palette,  # sns.color_palette("PuBuGn_d"),#["#9ecae1", "#cccccc"],
        hue=hue,
        dodge=dodge
        # palette=('Blues_d' if hue is not None else None)
    )
    plt.title(title, fontsize=14)
    plt.xlabel(xlabel, fontsize=12)
    plt.ylabel(ylabel, fontsize=12)

    for label in axes.xaxis.get_ticklabels():
        label.set_rotation(60)

    axes.yaxis.set_major_formatter(matplotlib.ticker.StrMethodFormatter("{x:,.0f}"))

    plt.show()


def get_edited_page_count(
    only_popular=False,
    period=properties.DATE_YEAR,
    desc="YEAR",
    plot=True,
    save: bool = False,
):
    periodic_df = processor.get_wikipedia_page_periodic_overview(
        only_popular=only_popular, period=period, desc=desc
    )
    dict_ = [
        {"period": period, "article_count": len(set(period_df["title"].unique()))}
        for period, period_df in periodic_df.groupby(["period"])
    ]

    result_df = pd.DataFrame(dict_)
    if plot:
        barplot(
            "period",
            "article_count",
            result_df,
            ylabel="Count of unique Wikipages",
            xlabel=desc.capitalize(),
            title="{}ly Count of Edited/Created Climate Change Wikipedia Pages".format(
                desc.capitalize()
            ),
        )

    if save:
        utils.create_folder(properties.STATS_FOLDER)
        name = "{}ly_count_edited-and-created_wikipages.csv".format(desc.lower())
        result_df.to_csv(f"{properties.STATS_FOLDER}/{name}")
    return result_df


# get_edited_page_count
def get_created_page_count(
    only_popular=False,
    period=properties.DATE_YEAR,
    desc="YEAR",
    plot=True,
    save: bool = False,
):
    periodic_df = processor.get_wikipedia_page_periodic_overview(
        only_popular=only_popular, period=period, desc=desc
    )

    articles = []
    result = []
    for period_, df_ in periodic_df.groupby("period"):
        period_articles = df_["title"].unique().tolist()
        count_new_articles = len([x for x in period_articles if x not in articles])
        articles.extend(period_articles)
        if len(articles) == 0:
            print("error")
        row = {"period": period_, "new_article_count": count_new_articles}
        result.append(row)
    result_df = pd.DataFrame(result)
    if plot:
        barplot(
            "period",
            "new_article_count",
            result_df,
            ylabel="Count of Created Wikipages",
            xlabel=desc.capitalize(),
            title="{}ly Count of Created Climate Change Wikipedia Pages".format(
                desc.capitalize()
            ),
        )

    if save:
        utils.create_folder(properties.STATS_FOLDER)
        name = "{}ly_count_created_wikipages.csv".format(desc.lower())
        result_df.to_csv(f"{properties.STATS_FOLDER}/{name}")
    return pd.DataFrame(result)


def get_revisions_over_time(
    only_popular=False,
    period=properties.DATE_YEAR,
    desc="YEAR",
    plot=True,
    save: bool = False,
):
    periodic_df = processor.get_wikipedia_page_periodic_overview(
        only_popular=only_popular, period=period, desc=desc
    )

    dict_ = [
        {"period": period_, "revision_count": df_["revision_count"].sum()}
        for period_, df_ in periodic_df.groupby(["period"])
    ]
    result_df = pd.DataFrame(dict_)

    if plot:
        barplot(
            "period",
            "revision_count",
            result_df,
            ylabel="Count of Wikipage Revisions",
            xlabel=desc.capitalize(),
            title="{}ly Count of Wikipedia Revisions".format(desc.capitalize()),
        )
    if save:
        utils.create_folder(properties.STATS_FOLDER)
        name = "{}ly_count_wikipedia_page_revisions.csv".format(desc.lower())
        result_df.to_csv(f"{properties.STATS_FOLDER}/{name}")
    return result_df


def get_words_over_time(
    only_popular=False,
    period=properties.DATE_YEAR,
    desc="YEAR",
    plot=True,
    save: bool = False,
):
    periodic_df = processor.get_wikipedia_page_periodic_overview(
        only_popular=only_popular, period=period, desc=desc
    )
    periodic_df.sort_values(by="timestamp", inplace=True)

    pre_period_content_dict = {}
    result_arr = []

    for period_, df_ in periodic_df.groupby(["period"]):
        current_title_content = {
            row["title"]: str(row["content"]) for _, row in df_.iterrows()
        }
        pre_period_content_dict.update(current_title_content)

        period_word_num = {
            "period": period_,
            "word_count": sum(
                [len(v.split()) for _, v in pre_period_content_dict.items()]
            ),
        }
        result_arr.append(period_word_num)

    result_df = pd.DataFrame(result_arr)
    if plot:
        barplot(
            "period",
            "word_count",
            result_df,
            ylabel="Count of Total Words",
            xlabel=desc.capitalize(),
            title="{}ly Wikipedia Page Word Count".format(desc.capitalize()),
        )
    if save:
        utils.create_folder(properties.STATS_FOLDER)
        name = "{}ly_count_wikipedia_page_words.csv".format(desc.lower())
        result_df.to_csv(f"{properties.STATS_FOLDER}/{name}")

    return result_df


def get_users_over_time(
    only_popular=False,
    period=properties.DATE_YEAR,
    desc="YEAR",
    plot=True,
    save: bool = False,
):
    periodic_df = processor.get_wikipedia_page_periodic_overview(
        only_popular=only_popular, period=period, desc=desc
    )
    result_arr = []
    over_time_anonymous = []
    over_time_registered = []
    for period_, df_ in periodic_df.groupby(["period"]):
        anonymous = []
        registered = []

        for user_lst_wiki in df_["users"].values.tolist():
            user_lst = ast.literal_eval(user_lst_wiki)

            anonymous.extend([x for x in user_lst if utils.is_ip(x)])
            registered.extend([x for x in user_lst if not utils.is_ip(x)])

        result_arr.append(
            {
                "period": period_,
                "count_edits": len(registered),
                "count_unique_users": len(set(registered)),
                "User Type": "Registered User",
            }
        )
        result_arr.append(
            {
                "period": period_,
                "count_edits": len(anonymous),
                "count_unique_users": len(set(anonymous)),
                "User Type": "Anonymous User",
            }
        )
        over_time_anonymous.extend(anonymous)
        over_time_registered.extend(registered)

    print("Total of Unique Anonymous Users:  ", len(set(over_time_anonymous)))
    print("Total of Unique Registered Users: ", len(set(over_time_registered)))

    result_df = pd.DataFrame(result_arr)
    if plot:
        barplot(
            "period",
            "count_edits",
            result_df,
            ylabel="Count of Revisions",
            xlabel=desc.capitalize(),
            title="{}ly edits count of Wikipedia Pages Per user type".format(
                desc.capitalize()
            ),
            hue="User Type",
            dodge=True,
        )
        barplot(
            "period",
            "count_unique_users",
            result_df,
            ylabel="Count of Unique Users",
            xlabel=desc.capitalize(),
            title="{}ly Count of Wikipedia Page Users".format(desc.capitalize()),
            hue="User Type",
            dodge=True,
        )

    if save:
        utils.create_folder(properties.STATS_FOLDER)
        name = "{}ly_count_wikipedia_page_users.csv".format(desc.lower())
        result_df.to_csv(f"{properties.STATS_FOLDER}/{name}")
    return result_df


def get_most_edited_articles(
    top=3, only_popular=False, period=properties.DATE_YEAR, desc="YEAR", plot=True
):
    sns.set(font_scale=1)
    plt.style.use("fivethirtyeight")
    periodic_df = processor.get_wikipedia_page_periodic_overview(
        only_popular=only_popular, period=period, desc=desc
    )
    num_period = len(periodic_df["period"].unique())
    f, axes = plt.subplots(1, num_period, sharex=False, sharey=True, figsize=(20, 6))

    idx = 0
    for period_, df_ in periodic_df.groupby(["period"]):
        top_titles_df = (
            df_[["title", "revision_count"]]
            .groupby("title")
            .sum()
            .sort_values(by="revision_count", ascending=False)
            .reset_index()[0:top]
        )

        ax = axes[idx]
        f.add_subplot(axes[idx])
        sns.barplot(
            x="title",
            y="revision_count",
            data=top_titles_df,
            color=sns.color_palette("PuBuGn_d", num_period)[idx],
        )

        # labels = [textwrap.fill(label.get_text(), 15) for label in ax.get_xticklabels()]
        # ax.set_xticklabels(labels)
        plt.gca().set_xticklabels(
            ax.get_xticklabels(), rotation=90, ha="right", fontsize=10
        )
        plt.gca().set_title(period_, fontsize=14)
        plt.gca().set_xlabel("")
        plt.gca().set_ylabel("")

        idx = idx + 1

    # plt.tight_layout()
    plt.subplots_adjust(wspace=1, hspace=2)
    plt.margins(x=0, y=0)
    # f.text(1, -0.32, 'Year', ha='center')
    f.text(
        0.5,
        1,
        "Top {} Titles per {}".format(top, desc.capitalize()),
        ha="center",
        fontsize=16,
    )
    plt.show()


def get_periodic_most_to_least_revised(
    only_popular=False, period=properties.DATE_YEAR, desc="YEAR", save: bool = False
):
    periodic_df = processor.get_wikipedia_page_periodic_overview(
        only_popular=only_popular, period=period, desc=desc
    )
    num_period = len(periodic_df["period"].unique())

    _dict = {}
    for period_, df_ in periodic_df.groupby(["period"]):
        sorted_titles = (
            df_[["title", "revision_count"]]
            .groupby("title")
            .sum()
            .sort_values(by="revision_count", ascending=False)
            .reset_index()
        )
        _dict[period_] = sorted_titles["title"].values.tolist()

    max = -1
    for _, v in _dict.items():
        max = len(v) if max < len(v) else max
    extended_dict = {k: v + ([""] * (max - len(v))) for k, v in _dict.items()}

    result_df = pd.DataFrame(extended_dict)
    if save:
        utils.create_folder(properties.STATS_FOLDER)
        name = "{}ly_most-to-least_revised_wikipedia_pages.csv".format(desc.lower())
        result_df.to_csv(f"{properties.STATS_FOLDER}/{name}")
    return result_df


def get_attr_for_period_as_txt(
    period_val,
    attr_val: str = "links",
    only_popular=False,
    period=properties.DATE_YEAR,
    desc="YEAR",
) -> []:  ## could be categories or urls
    periodic_df = processor.get_wikipedia_page_periodic_overview(
        only_popular=only_popular, period=period, desc=desc
    )
    periodic_df = periodic_df[periodic_df["period"] == period_val].copy()
    periodic_df.sort_values(
        by=["revision_count", "title"], inplace=True, ascending=False
    )

    result_arr = []
    for _, row in periodic_df.iterrows():
        lst_ = ast.literal_eval(row[attr_val])
        lst_ = [x.replace(" ", "") for x in lst_]
        if len(lst_) > 0:
            result_arr.append(". ".join(lst_))

    utils.create_folder(properties.STATS_FOLDER)
    with open(
        f"{properties.STATS_FOLDER}/{attr_val}_{period_val}.txt", "w", encoding="utf-8"
    ) as f:
        for line in result_arr:
            f.write(f"{line}\n")
    return result_arr


def get_periodic_revisions_percentage(
    only_popular=False, period=properties.DATE_YEAR, desc="YEAR", save: bool = False
):
    periodic_df = processor.get_wikipedia_page_periodic_overview(
        only_popular=only_popular, period=period, desc=desc
    )
    periodic_df.sort_values(
        by=["period", "revision_count", "title"], inplace=True, ascending=False
    )

    periodic_df["revision_percentage"] = round(
        100
        * periodic_df["revision_count"]
        / periodic_df.groupby("period")["revision_count"].transform("sum"),
        3,
    )
    result_df = periodic_df[
        ["period", "title", "revision_count", "revision_percentage", "timestamp"]
    ]
    if save:
        utils.create_folder(properties.STATS_FOLDER)
        name = "{}ly_wikipedia_page_revisions_w_percentage.csv".format(desc.lower())
        result_df.to_csv(f"{properties.STATS_FOLDER}/{name}")

    return result_df



import logging

from wikipedia_tools.utils import properties
from wikipedia_tools.utils import utils
from pathlib import Path
from datetime import datetime

import pandas as pd
from glob import glob
import os


def get_wikipedia_page_titles(
    categories: list = ["Climate_change"], corpus: str = properties.CORPUS
):
    wikipedia_pages_df = {}

    for category in categories:
        try:
            all_unique_pages = [
                Path(x).name.replace(".parquet", "")
                for x in glob(os.path.join(corpus, category, "*", "*.parquet"))
            ]
            all_unique_pages = list(set(all_unique_pages))

            wikipedia_pages_df[category] = all_unique_pages
            logging.info(
                f"There are {len(all_unique_pages)} wikipedia pages for the category {category}"
            )
        except:
            logging.error(
                f"Make sure that the corpus parquet files are located here {os.path.join(corpus, category)}"
            )
    return wikipedia_pages_df


def get_wikipedia_page_data(
    wikipedia_page: str,
    category: str = "Climate_change",
    corpus: str = properties.CORPUS,
) -> pd.DataFrame:
    def to_tuple(arr):
        return tuple(map(tuple, arr))

    paths = glob(os.path.join(corpus, category, "*", f"{wikipedia_page}.parquet"))
    data_df: pd.DataFrame = None
    _all_data_lst = []
    for path in paths:
        _df = pd.read_parquet(path)

        _df["categories"] = _df["categories"].apply(tuple)
        _df["images"] = _df["images"].apply(tuple)
        _df["links"] = _df["links"].apply(tuple)
        _df["urls"] = _df["urls"].apply(tuple)
        _df["sections"] = _df["sections"].apply(to_tuple)

        _all_data_lst.append(_df)

    if len(_all_data_lst) == 1:
        data_df = _all_data_lst[0]
    elif len(_all_data_lst) > 1:
        data_df = pd.concat(_all_data_lst)
        data_df.drop_duplicates(keep="first", inplace=True)

    return data_df



import datetime
import logging

import pandas as pd
import os

from glob import glob
from tqdm import tqdm

from wikipedia_tools.utils import properties, utils
from wikipedia_tools.processor import loader
from pathlib import Path


def get_wiki_popular_pages():
    wiki_popular_page_df = pd.read_csv(properties.WIKI_CLIMATE_CHANGE_POPULAR_PAGES)
    return wiki_popular_page_df[wiki_popular_page_df["Page title"].notna()]


def is_popular(title):
    popular_df = get_wiki_popular_pages()
    return len(popular_df[popular_df["Page title"] == title]) > 0


def is_important(title):
    popular_df = get_wiki_popular_pages()
    return (
        popular_df[popular_df["Page title"] == title]["Importance"].values[0]
        if is_popular(title)
        else "na"
    )


def get_wiki_pages_overview(category="Climate_change"):
    """
    Processes all wikipages and generates 1 dataframe where each row represents 1 wikipage with info:
    page title, created on, last updated, initial content, latest content, edits numner, content edits count, unique users,
    is_popular (if it exists under Wikipedida Climate Change Project), is_important (if it is popular and flagged important by the wiki project)
    :return:
    """
    path = Path(properties.WIKI_OVERVIEW_CSV)
    if path.is_file():
        return pd.read_csv(
            properties.WIKI_OVERVIEW_CSV,
        )

    wiki_titles = loader.get_wikipedia_page_titles(
        categories=[category], corpus=properties.CORPUS
    )
    popular_pages_df = get_wiki_popular_pages()
    wiki_page_meta_arr = []

    print("there are {} wiki titles".format(len(wiki_titles)))
    for title_code in tqdm(wiki_titles[category]):
        original_df = loader.get_wikipedia_page_data(title_code)

        if original_df is None:
            continue
        if len(original_df) == 0:
            print("{} has len 0".format(title_code))
            continue
        _df = original_df.copy()

        wiki_page_meta = {}

        _df["timestamp_str"] = _df["timestamp"]
        _df["timestamp"] = pd.to_datetime(_df["timestamp"], infer_datetime_format=True)
        _df = _df.sort_values(by="timestamp")

        init_content_df = _df[(~_df["content"].isna()) & (_df["content"] != "")].copy()
        init_content_df = init_content_df[
            init_content_df["timestamp"] == init_content_df["timestamp"].min()
        ]
        if len(init_content_df) == 0:
            print("NO CONTENT: ", title_code)
            continue
        last_content = _df[_df["timestamp"] == _df["timestamp"].max()][
            "content"
        ].values[0]
        edits_w_latest_content_df = _df[(_df["content"] == last_content)]
        last_updated_df = edits_w_latest_content_df[
            edits_w_latest_content_df["timestamp"]
            == edits_w_latest_content_df["timestamp"].min()
        ]

        wiki_page_meta["title_code"] = title_code
        wiki_page_meta["page_title"] = init_content_df["page"].values[0]
        wiki_page_meta["created_on"] = init_content_df["timestamp"].values[0]
        wiki_page_meta["init_content"] = init_content_df["content"].values[0]
        wiki_page_meta["last_updated"] = last_updated_df["timestamp"].values[0]
        wiki_page_meta["latest_content"] = last_updated_df["content"].values[0]
        wiki_page_meta["edits_number"] = len(original_df["timestamp"].unique())
        wiki_page_meta["content_edits_number"] = len(
            _df.drop_duplicates(subset=["content"], keep="first", ignore_index=True)
        )
        wiki_page_meta["unique_users"] = len(original_df["user"].unique())

        wiki_page_meta["is_popular"] = is_popular(init_content_df["page"].values[0])
        wiki_page_meta["importance"] = (
            popular_pages_df[popular_pages_df["Page title"] == title_code][
                "Importance"
            ].values[0]
            if wiki_page_meta["is_popular"]
            else "na"
        )

        wiki_page_meta_arr.append(wiki_page_meta)

    meta_df = pd.DataFrame(wiki_page_meta_arr)
    meta_df.to_csv(properties.WIKI_OVERVIEW_CSV, index=False)
    return meta_df


# get_wiki_edits_num
def get_revision_count(
    only_popular=False,
    date_granularity=properties.DATE_MONTHLY,
    desc="MONTH-YEAR",
    category="Climate_change",
):
    fname = properties.WIKI_EDITS_COUNT_CSV.format(
        ("POPULAR" if only_popular else "ALL"), desc
    )
    path = Path(fname)
    if path.is_file():
        return pd.read_csv(fname, index_col="abstracted_date")

    wiki_titles = loader.get_wikipedia_page_titles(
        categories=[category], corpus=properties.CORPUS
    )

    print("there are {} wiki titles".format(len(wiki_titles)))

    result_df = pd.DataFrame()

    for title_code in tqdm(wiki_titles[category]):
        original_df = loader.get_wikipedia_page_data(title_code)
        title = original_df["page"].values[0]
        if (
            (only_popular and not is_popular(title))
            or (original_df is None)
            or len(original_df) == 0
        ):
            continue

        _df = original_df.copy()

        _df["timestamp_str"] = _df["timestamp"]
        _df["timestamp"] = pd.to_datetime(_df["timestamp"], infer_datetime_format=True)
        _df["abstracted_date"] = _df["timestamp"].dt.strftime(date_granularity)

        edits_count_df = (
            _df["abstracted_date"].value_counts().to_frame(name="edits_count")
        )
        edits_count_df.index.name = "abstracted_date"
        result_df = (
            pd.concat([result_df, edits_count_df]).groupby(["abstracted_date"]).sum()
        )

    result_df.reset_index().to_csv(fname, index=False)
    return result_df


#


def batch_revisions_per_period(
    period=properties.DATE_MONTHLY,
    desc="monthly",
    category="Climate_change",
    corpus=None,
    destination: str = ""
):

    folder_root = properties.FOLDER_WIKI_BATCHES.format(destination, desc)
                    
    corpus = properties.CORPUS if corpus is None else corpus
    wiki_titles = loader.get_wikipedia_page_titles(categories=[category],
                                                   corpus=corpus)

    print("there are {} wiki titles".format(len(wiki_titles)))
    periords_str_lst = []
    for title_code in tqdm(wiki_titles[category]):
        original_df = loader.get_wikipedia_page_data(title_code)
        if (original_df is None) or len(original_df) == 0:
            continue
        _df = original_df.copy()

        _df["timestamp_str"] = _df["timestamp"]
        _df["timestamp"] = pd.to_datetime(_df["timestamp"], infer_datetime_format=True)
        _df["abstracted_date"] = _df["timestamp"].dt.strftime(period)
        periords_str_lst.extend(_df["abstracted_date"].unique().tolist())
        for period_val, period_df in _df.groupby("abstracted_date"):
            destination = f"{folder_root}/{period_val}/{title_code}.parquet"
            utils.create_folder(f"{folder_root}/{period_val}")
            period_df.to_parquet(destination, index=False)
    periods_str_lst = list(set(periords_str_lst))
    periods_str_lst = sorted([datetime.strptime(x, period) for x in periods_str_lst])

    return folder_root, periods_str_lst


# get_wiki_edits_per_period
def get_wikipedia_page_periodic_overview(
    only_popular=False,
    period=properties.DATE_YEAR,
    desc="YEAR",
    category="Climate_change",
):
    fname = properties.WIKI_EDITS_PERIODIC_CSV.format(
        ("POPULAR" if only_popular else "ALL"), desc
    )
    path = Path(fname)
    if path.is_file():
        return pd.read_csv(fname)

    wiki_titles = loader.get_wikipedia_page_titles(
        categories=[category], corpus=properties.CORPUS
    )

    print("there are {} wiki titles".format(len(wiki_titles)))

    result_arr = []
    for title_code in tqdm(wiki_titles[category]):
        original_df = loader.get_wikipedia_page_data(title_code)
        if (
            (only_popular and not is_popular(original_df["page"].values[0]))
            or (original_df is None)
            or len(original_df) == 0
        ):
            continue
        _df = original_df.copy()

        _df["timestamp_str"] = _df["timestamp"]
        _df["timestamp"] = pd.to_datetime(_df["timestamp"], infer_datetime_format=True)
        _df["abstracted_date"] = _df["timestamp"].dt.strftime(period)

        for period_val, period_df in _df.groupby("abstracted_date"):
            latest_df = period_df[
                period_df["timestamp"] == period_df["timestamp"].max()
            ]
            revision_per_period = {}
            revision_per_period["content"] = latest_df["content"].values[0]
            revision_per_period["title"] = latest_df["page"].values[0]
            revision_per_period["title_code"] = title_code
            revision_per_period["period"] = period_val
            revision_per_period["user_count"] = len(period_df["user"].unique())
            revision_per_period["users"] = period_df["user"].values.tolist()
            revision_per_period["revision_count"] = len(period_df)
            revision_per_period["timestamp"] = latest_df["timestamp"].values[0]
            revision_per_period["links"] = latest_df["links"].values[0]
            revision_per_period["categories"] = latest_df["categories"].values[0]
            revision_per_period["urls"] = latest_df["urls"].values[0]
            result_arr.append(revision_per_period)

    result_df = pd.DataFrame(result_arr)  # , index = ['period', 'title'])

    result_df.to_csv(fname, index=False)
    return result_df
